
%define kernelversion	3
%define patchlevel	10
# sublevel is now used for -stable patches
%define sublevel	28

# Linux-VServer version
%define	vsversion	2.3.6.8

# kernel Makefile extraversion is substituted by
# kpatch/kgit/kstable wich are either 0 (empty), rc (kpatch), git (kgit)
# or stable release (kstable04)
%define kpatch		0

# kernel.org -gitX patch (only the number after "git")
%define kgit		0

# this is the releaseversion
%define mgarelease	0.vs%{vsversion}.1

# This is only to make life easier for people that creates derivated kernels
# a.k.a name it kernel-tmb :)
%define ktag		vserver
%define kname		kernel-%{ktag}

%define rpmtag		%{distsuffix}%{mgaver}
%if %kpatch
%if %kgit
%define rpmrel		%mkrel 0.%{kpatch}.%{kgit}.%{mgarelease}
%else
%define rpmrel		%mkrel 0.%{kpatch}.%{mgarelease}
%endif
%else
%define rpmrel		%mkrel %{mgarelease}
%endif

# theese two never change, they are used to fool rpm/urpmi/smart
%define fakever		1
%define fakerel		%mkrel 1

# When we are using a pre/rc patch, the tarball is a sublevel -1
%if %kpatch
%define kversion	%{kernelversion}.%{patchlevel}.%{sublevel}
%if %sublevel
%define tar_ver		%{kernelversion}.%{patchlevel}
%else
%define tar_ver		%{kernelversion}.%(expr %{patchlevel} - 1)
%endif
%else
%define kversion	%{kernelversion}.%{patchlevel}.%{sublevel}
%define tar_ver		%{kernelversion}.%{patchlevel}
%endif
%define kverrel		%{kversion}-%{rpmrel}

# used for not making too long names for rpms or search paths
%if %kpatch
%if %kgit
%define buildrpmrel	0.%{kpatch}.%{kgit}.%{mgarelease}%{rpmtag}
%else
%define buildrpmrel	0.%{kpatch}.%{mgarelease}%{rpmtag}
%endif
%else
%define buildrpmrel	%{mgarelease}%{rpmtag}
%endif

%define buildrel	%{kversion}-%{buildrpmrel}

%define kvserver_notice NOTE: This kernel is only patched with vserver patch.

# having different top level names for packges means that you have to remove them by hard :(
%define top_dir_name    %{kname}-%{_arch}

%define build_dir       ${RPM_BUILD_DIR}/%{top_dir_name}
%define src_dir         %{build_dir}/linux-%{tar_ver}

# disable useless debug rpms...
%define _enable_debug_packages  %{nil}
%define debug_package           %{nil}

# build defines
%define build_doc 1
%define build_source 1
%define build_devel 1

%define build_kernel 1

# End of user definitions
%{?_without_kernel: %global build_kernel 0}
%{?_without_doc: %global build_doc 0}
%{?_without_source: %global build_source 0}
%{?_without_devel: %global build_devel 0}

%{?_with_kernel: %global build_kernel 1}
%{?_with_doc: %global build_doc 1}
%{?_with_source: %global build_source 1}
%{?_with_devel: %global build_devel 1}

%if %(if [ -z "$CC" ] ; then echo 0; else echo 1; fi)
%define kmake %make CC="$CC"
%else
%define kmake %make
%endif
# there are places where parallel make don't work
%define smake make

# Parallelize xargs invocations on smp machines
%define kxargs xargs %([ -z "$RPM_BUILD_NCPUS" ] \\\
	&& RPM_BUILD_NCPUS="`/usr/bin/getconf _NPROCESSORS_ONLN`"; \\\
	[ "$RPM_BUILD_NCPUS" -gt 1 ] && echo "-P $RPM_BUILD_NCPUS")

# Aliases for amd64 builds (better make source links?)
%define target_cpu	%(echo %{_target_cpu} | sed -e "s/amd64/x86_64/")
%define target_arch	%(echo %{_arch} | sed -e "s/amd64/x86_64/")

# src.rpm description
Summary: 	The Linux kernel (the core of the Linux operating system)
Name:           %{kname}
Epoch:		1
Version:        %{kversion}
Release:        %{rpmrel}
License: 	GPLv2
Group: 		System/Kernel and hardware
ExclusiveArch: 	%{ix86} x86_64
ExclusiveOS: 	Linux
URL: 		http://www.kernel.org/

####################################################################
#
# Sources
#
### This is for full SRC RPM
Source0:        ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/linux-%{tar_ver}.tar.xz
Source1:        ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/linux-%{tar_ver}.tar.sign

# This is for disabling mrproper and other targets on -devel rpms
Source2:	disable-mrproper-in-devel-rpms.patch

Source4:  	README.kernel-sources

# Kernel defconfigs
Source20: 	i386_defconfig
Source21: 	x86_64_defconfig
Source22: 	arm_defconfig


####################################################################
#
# Patches

#
# Patch0 to Patch100 are for core kernel upgrades.
#

# Pre linus patch: ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/testing

%if %sublevel
%if %kpatch
%if %sublevel > 1
Patch1:		ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/patch-%{kernelversion}.%{patchlevel}.%(expr %{sublevel} - 1).xz
Source11:	ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/patch-%{kernelversion}.%{patchlevel}.%(expr %{sublevel} - 1).sign
%endif
Patch2:		ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/stable-review//patch-%{kernelversion}.%{patchlevel}.%{sublevel}-%{kpatch}.xz
Source12:	ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/stable-review/patch-%{kernelversion}.%{patchlevel}.%{sublevel}-%{kpatch}.sign
%else
Patch3:		ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/testing/patch-%{kernelversion}.%{patchlevel}.%{sublevel}.xz
Source13:	ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/testing/patch-%{kernelversion}.%{patchlevel}.%{sublevel}.sign
%endif
%else
%if %sublevel
Patch4:		ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/patch-%{kversion}.xz
Source14:	ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/patch-%{kversion}.sign
%endif
%endif
# kernel.org -git
%if %kgit
Patch5:		ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/snapshots/patch-%{kernelversion}.%{patchlevel}-%{kpatch}-git%{kgit}.xz
Source15:	ftp://ftp.kernel.org/pub/linux/kernel/v%{kernelversion}.%{patchlevel}/snapshots/patch-%{kernelversion}.%{patchlevel}-%{kpatch}-git%{kgit}.sign
%endif

# -stable queue

# CVEs
Patch14:	fs-xfs-add-capability-check-to-free-eofblocks-ioctl.patch
Patch15:	x86-x32-Correct-invalid-use-of-user-timespec-in-the-kernel_CVE-2014-0038.patch

# -vserver patch
Patch40:	http://vserver.13thfloor.at/Experimental/patch-%{kernelversion}.%{patchlevel}.%{sublevel}-vs%{vsversion}.diff

### other fixes

# nice speedup
Patch50:	cpufreq-ondemand-Change-the-calculation-of-target-frequency.patch

#END
####################################################################

# Defines for the things that are needed for all the kernels
%define requires1       bootloader-utils >= 1.16-1
%define requires2       dracut >= 025-8
%define requires3       kmod >= 12-2
%define requires4       sysfsutils >= 2.1.0-16
%define requires5       kernel-firmware >= 20130624-1

%define kprovides kernel = %{tar_ver}, alsa

# conflict dkms packages that dont support kernel-3.10
%define kconflicts1     dkms-broadcom-wl < 5.100.82.112-13
%define kconflicts1     dkms-fglrx < 13.101-0.3
%define kconflicts2     dkms-nvidia-current < 319.32-2
%define kconflicts3     dkms-nvidia304 < 304.88-4
%define kconflicts4     dkms-nvidia173 < 173.14.37-4
# (tmb) nvidia96xx does not support this kernel or x11-server-1.13
%define kconflicts5     dkms-nvidia96xx <= 96.43.23
%define kconflicts6     dmms-xtables-addons < 2.3-1
# (tmb) conflict too old lvm2 rules to make upgrades work (tv, mga #9536)
%define kconflicts7     lvm2 < 2.02.98-3
# (tmb) conflict too old drakxtools-backend to avoid bogus root=/dev/ (#9536)
%define kconflicts8     drakxtools-backend < 15.37-1
# (tmb) conflict older btrfs-progs to get the new in same transaction and in initrd
%define kconflicts9     btrfs-progs < 0.20-0.rc1.20130607.2
# (tmb) conflict too old plymouth (#10128)
%define kconflicts10    plymouth < 0.8.6.1-8
# (tmb) conflict too old radeon-firmware to get the uvd firmwares in initrd
%define kconflicts11    radeon-firmware < 20130626-1

Autoreqprov: 	no
BuildRequires: 	gcc kmod >= 7-7 bc

%description
Source package to build the Linux-VServer kernel.

%{kvserver_notice}


#
# kernel: Symmetric MultiProcessing kernel
#
%if %build_kernel
%package -n %{kname}-%{buildrel}
Version:	%{fakever}
Release:	%{fakerel}
%ifarch %{ix86}
Summary:	Linux Kernel for vserver use with i586 & 4GB RAM
%else
Summary:	Linux Kernel for vserver use with %{_arch}
%endif
Group:		System/Kernel and hardware
Provides:	%kprovides
Provides:	should-restart = system
Requires:       %requires1
Requires:       %requires2
Requires:       %requires3
Requires:       %requires4
Requires:       %requires5
Conflicts:      %kconflicts1
Conflicts:      %kconflicts2
Conflicts:      %kconflicts3
Conflicts:      %kconflicts4
Conflicts:      %kconflicts5
Conflicts:      %kconflicts6
Conflicts:      %kconflicts7
Conflicts:      %kconflicts8
Conflicts:      %kconflicts9
Conflicts:      %kconflicts10
Conflicts:      %kconflicts11

%ifarch %{ix86}
Conflicts:	arch(x86_64)
%endif

%description -n %{kname}-%{buildrel}
%ifarch %{ix86}
This kernel is compiled for vserver use, single or multiple i586
processor(s)/core(s) and a maximum of 3GB RAM, using HZ_1000, no
preempt, CFS cpu scheduler and cfq i/o scheduler.
This kernel relies on in-kernel smp alternatives to switch between
up & smp mode depending on detected hardware. To force the kernel
to boot in single processor mode, use the "nosmp" boot parameter.
%else
This kernel is compiled for vserver use, single or multiple %{_arch}
processor(s)/core(s), using HZ_1000, no preempt, CFS cpu
scheduler and cfq i/o scheduler.
This kernel relies on in-kernel smp alternatives to switch between
up & smp mode depending on detected hardware. To force the kernel
to boot in single processor mode, use the "nosmp" boot parameter.
%endif

%{kvserver_notice}
%endif # build_kernel


#
# kernel-source: kernel sources
#
%if %build_source
%package -n %{kname}-source-%{buildrel}
Version:	%{fakever}
Release:	%{fakerel}
Provides:	%{kname}-source, kernel-source = %{kverrel}
Provides:	%{kname}-source-%{kernelversion}.%{patchlevel}
Requires:	glibc-devel, ncurses-devel, make, gcc, perl, diffutils
Summary:	The source code for %{kname}
Group:		Development/Kernel
Autoreqprov: 	no
Buildarch:	noarch

%description -n %{kname}-source-%{buildrel}
The %{kname}-source package contains the source code files for the
Linux-VServer kernel. Theese source files are only needed if you want to 
build your own custom kernel that is better tuned to your particular hardware.

If you only want the files needed to build 3rdparty (nVidia, Ati, dkms-*,...)
drivers against, install the *-devel-* rpm that is matching your kernel.

%{kvserver_notice}
%endif #build_source


#
# kernel-devel: stripped kernel sources
#
%if %build_devel
%package -n %{kname}-devel-%{buildrel}
Version:	%{fakever}
Release:	%{fakerel}
Provides:	kernel-devel = %{kverrel}
Summary:	The %{kname} devel files for 3rdparty modules build
Group:		Development/Kernel
Autoreqprov:	no
Requires:	glibc-devel, ncurses-devel, make, gcc, perl
%ifarch %{ix86}
Conflicts:	arch(x86_64)
%endif

%description -n %{kname}-devel-%{buildrel}
This package contains the kernel-devel files that should be enough to build
3rdparty drivers against for use with the %{kname}-%{buildrel}.

If you want to build your own kernel, you need to install the full
%{kname}-source-%{buildrel} rpm.

%{kvserver_notice}
%endif #build_devel


#
# kernel-doc: documentation for the Linux kernel
#
%if %build_doc
%package -n %{kname}-doc
Version:        %{kversion}
Release:        %{rpmrel}
Summary:	Various documentation bits found in the kernel source
Group:		Documentation
Buildarch:	noarch

%description -n %{kname}-doc
This package contains documentation files form the kernel source. Various
bits of information about the Linux-VServer kernel and the device drivers 
shipped with it are documented in these files. You also might want install 
this package if you need a reference to the options that can be passed to 
Linux kernel modules at load time.

%{kvserver_notice}
%endif #build_doc


#
# kernel-latest: virtual rpm
#
%if %build_kernel
%package -n %{kname}-latest
Version:        %{kversion}
Release:        %{rpmrel}
Summary: 	Virtual rpm for latest %{kname}
Group: 	  	System/Kernel and hardware
Requires: 	%{kname}-%{buildrel}
Obsoletes:	%{kname}-smp-latest
%ifarch %{ix86}
Conflicts:	arch(x86_64)
%endif

%description -n %{kname}-latest
This package is a virtual rpm that aims to make sure you always have the
latest %{kname} installed...

%{kvserver_notice}
%endif #build_kernel


#
# kernel-source-latest: virtual rpm
#
%if %build_source
%package -n %{kname}-source-latest
Version:        %{kversion}
Release:        %{rpmrel}
Summary: 	Virtual rpm for latest %{kname}-source
Group: 	  	System/Kernel and hardware
Requires: 	%{kname}-source-%{buildrel}
Buildarch:	noarch

%description -n %{kname}-source-latest
This package is a virtual rpm that aims to make sure you always have the
latest %{kname}-source installed...

%{kvserver_notice}
%endif #build_source


#
# kernel-devel-latest: virtual rpm
#
%if %build_devel
%package -n %{kname}-devel-latest
Version:        %{kversion}
Release:        %{rpmrel}
Summary: 	Virtual rpm for latest %{kname}-devel
Group: 	  	System/Kernel and hardware
Requires: 	%{kname}-devel-%{buildrel}
Obsoletes:	%{kname}-smp-devel-latest
Obsoletes:	%{kname}-smp-headers-latest
%ifarch %{ix86}
Conflicts:	arch(x86_64)
%endif

%description -n %{kname}-devel-latest
This package is a virtual rpm that aims to make sure you always have the
latest %{kname}-devel installed...

%{kvserver_notice}
%endif #build_devel


#
# End packages - here begins build stage
#
%prep
%setup -q -n %top_dir_name -c

pushd %src_dir
%if %sublevel
%if %kpatch
%if %sublevel > 1
%patch1 -p1
%endif
%patch2 -p1
%else
%patch3 -p1
%endif
%else
%if %sublevel
%patch4 -p1
%endif
%endif
%if %kgit
%patch5 -p1
%endif

# stable queue

# CVEs
%patch14 -p1
%patch15 -p1

# -vserver
%patch40 -p1

### other fixes
# nice speedup
%patch50 -p1

popd

# PATCH END


#
# Setup Begin
#


# Install defconfigs...
install %{SOURCE20} %{build_dir}/linux-%{tar_ver}/arch/x86/configs/
install %{SOURCE21} %{build_dir}/linux-%{tar_ver}/arch/x86/configs/
install %{SOURCE22} %{build_dir}/linux-%{tar_ver}/arch/arm/configs/

# make sure the kernel has the sublevel we know it has...
LC_ALL=C perl -p -i -e "s/^SUBLEVEL.*/SUBLEVEL = %{sublevel}/" linux-%{tar_ver}/Makefile


%build
# Common target directories
%define _bootdir /boot
%define _modulesdir /lib/modules
%define _kerneldir /usr/src/linux-%{kversion}-%{ktag}-%{buildrpmrel}
%define _develdir /usr/src/linux-%{kversion}-%{ktag}-devel-%{buildrpmrel}


# Directories definition needed for building
%define temp_root %{build_dir}/temp-root
%define temp_boot %{temp_root}%{_bootdir}
%define temp_modules %{temp_root}%{_modulesdir}
%define temp_source %{temp_root}%{_kerneldir}
%define temp_devel %{temp_root}%{_develdir}


# Create a simulacro of buildroot
rm -rf %{temp_root}
install -d %{temp_root}


# make sure we are in the directory
cd %{src_dir}

# make sure EXTRAVERSION says what we want it to say
LC_ALL=C perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{ktag}-%{buildrpmrel}/" Makefile

# Prepare the kernel
%smake -s mrproper
%ifarch %{ix86} x86_64
	cp arch/x86/configs/%{target_arch}_defconfig .config
%else
	cp arch/%{target_arch}/defconfig .config
%endif
%smake oldconfig

# Build the kernel
%kmake -s all

# Install kernel
install -d %{temp_boot}
install -m 644 System.map \
               %{temp_boot}/System.map-%{kversion}-%{ktag}-%{buildrpmrel}
install -m 644 .config %{temp_boot}/config-%{kversion}-%{ktag}-%{buildrpmrel}
cp -f arch/%{target_arch}/boot/bzImage \
      %{temp_boot}/vmlinuz-%{kversion}-%{ktag}-%{buildrpmrel}

# Install modules
install -d %{temp_modules}/%{kversion}-%{ktag}-%{buildrpmrel}
%smake INSTALL_MOD_PATH=%{temp_root} \
       KERNELRELEASE=%{kversion}-%{ktag}-%{buildrpmrel} \
       modules_install

# remove /lib/firmware, we use a separate kernel-firmware
rm -rf %{temp_root}/lib/firmware

# Save devel tree
%if %build_devel
mkdir -p %{temp_devel}
for i in $(find . -name 'Makefile*'); do cp -R --parents $i %{temp_devel};done
for i in $(find . -name 'Kconfig*' -o -name 'Kbuild*' -o -name config.mk); do cp -R --parents $i %{temp_devel};done
cp -fR include %{temp_devel}
cp -fR scripts %{temp_devel}
cp -fR tools/include %{temp_devel}/tools/
%ifarch %{arm}
       cp -fR arch/%{target_arch}/tools $TempDevelRoot/arch/%{target_arch}/
%endif
%ifarch %{ix86} x86_64
	cp -fR arch/x86/kernel/asm-offsets.{c,s} %{temp_devel}/arch/x86/kernel/
	cp -fR arch/x86/kernel/asm-offsets_{32,64}.c %{temp_devel}/arch/x86/kernel/
	cp -fR arch/x86/syscalls/syscall_{32,64}.tbl %{temp_devel}/arch/x86/syscalls/
	cp -fR arch/x86/syscalls/syscall{tbl,hdr}.sh %{temp_devel}/arch/x86/syscalls/
	cp -fR arch/x86/include %{temp_devel}/arch/x86/
	cp -fR arch/x86/tools %{temp_devel}/arch/x86/
%else
	cp -fR arch/%{target_arch}/kernel/asm-offsets.{c,s} %{temp_devel}/arch/%{target_arch}/kernel/
	cp -fR arch/%{target_arch}/include %{temp_devel}/arch/%{target_arch}/
%endif

# Needed for generation of kernel/bounds.s
cp -fR kernel/bounds.c %{temp_devel}/kernel/

# Needed for lguest
cp -fR drivers/lguest/lg.h %{temp_devel}/drivers/lguest/

cp -fR .config Module.symvers %{temp_devel}

# Needed for truecrypt build (Danny)
cp -fR drivers/md/dm.h %{temp_devel}/drivers/md/

# Needed for external dvb tree (#41418)
cp -fR drivers/media/dvb-core/*.h %{temp_devel}/drivers/media/dvb-core/
cp -fR drivers/media/dvb-frontends/lgdt330x.h %{temp_devel}/drivers/media/dvb-frontends/

# add acpica header files, needed for fglrx build
cp -fR drivers/acpi/acpica/*.h %{temp_devel}/drivers/acpi/acpica/

# Check and clean the -devel tree
pushd %{temp_devel} >/dev/null
    %smake -s prepare scripts clean
    rm -f .config.old
popd >/dev/null

# Disable mrproper and other targets
patch -p1 -d %{temp_devel} -i %{SOURCE2}

# Fix permissions
chmod -R a+rX %{temp_devel}
%endif # build_devel

#make sure we are in the directory
cd %src_dir

# kernel-source is shipped as an unprepared tree
%smake -s mrproper


###
### Install
###
%install
install -m 644 %{SOURCE4}  .

cd %src_dir
# Directories definition needed for installing
%define target_source %{buildroot}/%{_kerneldir}
%define target_boot %{buildroot}%{_bootdir}
%define target_modules %{buildroot}%{_modulesdir}
%define target_devel %{buildroot}%{_develdir}

# We want to be able to test several times the install part
rm -rf %{buildroot}
cp -a %{temp_root} %{buildroot}

# Create directories infastructure
%if %build_source
install -d %{target_source}

tar cf - . | tar xf - -C %{target_source}
chmod -R a+rX %{target_source}

# we remove all the source files that we don't ship

# first architecture files
for i in alpha arc arm arm64 avr32 blackfin cris c6x frv hexagon h8300 ia64 m32r mips microblaze \
	m68k m68knommu metag mn10300 openrisc parisc powerpc ppc s390 score \
	sh sh64 sparc tile unicore32 v850 xtensa; do
	rm -rf %{target_source}/arch/$i

%if %build_devel
	rm -rf %{target_devel}/arch/$i
%endif
done

# remove arch files based on target arch
%ifnarch %{ix86} x86_64
	rm -rf %{target_source}/arch/x86
%if %build_devel
	rm -rf %{target_devel}/arch/x86
%endif
%endif


# other misc files
rm -f %{target_source}/{.config.old,.config.cmd,.tmp_gas_check,.mailmap,.missing-syscalls.d,arch/.gitignore}
rm -rf %{target_source}/.tmp_depmod/

#endif %build_source
%endif


# compressing modules
find %{target_modules} -name "*.ko" | %kxargs xz -6e


# We used to have a copy of PrepareKernel here
# Now, we make sure that the thing in the linux dir is what we want it to be

for i in %{target_modules}/*; do
  rm -f $i/build $i/source
done


# sniff, if we gzipped all the modules, we change the stamp :(
# we really need the depmod -ae here
pushd %{target_modules}
for i in *; do
	/sbin/depmod -ae -b %{buildroot} -F %{target_boot}/System.map-$i $i
	echo $?
done

for i in *; do
	pushd $i
	echo "Creating module.description for $i"
	modules=`find . -name "*.ko.xz"`
	echo $modules | %kxargs /sbin/modinfo \
	| perl -lne 'print "$name\t$1" if $name && /^description:\s*(.*)/; $name = $1 if m!^filename:\s*(.*)\.k?o!; $name =~ s!.*/!!' > modules.description
	popd
done
popd


###
### Clean
###

%clean
rm -rf %{buildroot}
# We don't want to remove this, the whole reason of its existence is to be 
# able to do several rpm --short-circuit -bi for testing install 
# phase without repeating compilation phase
#rm -rf %{temp_root} 


###
### Scripts
###

### kernel
%if %build_kernel
%preun -n %{kname}-%{buildrel}
/sbin/installkernel -R %{kversion}-%{ktag}-%{buildrpmrel}
if [ -L /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/build ]; then
    rm -f /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/build
fi
if [ -L /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/source ]; then
    rm -f /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/source
fi
pushd /boot > /dev/null
if [ -L vmlinuz-%{ktag} ]; then
    if [ "$(readlink vmlinuz-%{ktag})" = "vmlinuz-%{kversion}-%{ktag}-%{buildrpmrel}" ]; then
	rm -f vmlinuz-%{ktag}
    fi
fi
if [ -L initrd-%{ktag}.img ]; then
    if [ "$(readlink initrd-%{ktag}.img)" = "initrd-%{kversion}-%{ktag}-%{buildrpmrel}.img" ]; then
	rm -f initrd-%{ktag}.img
    fi
fi
popd > /dev/null
exit 0

%post -n %{kname}-%{buildrel}
/sbin/installkernel -L %{kversion}-%{ktag}-%{buildrpmrel}
if [ -d /usr/src/linux-%{kversion}-%{ktag}-devel-%{buildrpmrel} ]; then
    ln -sf /usr/src/linux-%{kversion}-%{ktag}-devel-%{buildrpmrel} \
           /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/build
    ln -sf /usr/src/linux-%{kversion}-%{ktag}-devel-%{buildrpmrel} \
           /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/source
fi
pushd /boot > /dev/null
if [ -L vmlinuz-%{ktag} ]; then
    rm -f vmlinuz-%{ktag}
fi
ln -sf vmlinuz-%{kversion}-%{ktag}-%{buildrpmrel} vmlinuz-%{ktag}
if [ -L initrd-%{ktag}.img ]; then
    rm -f initrd-%{ktag}.img
fi
ln -sf initrd-%{kversion}-%{ktag}-%{buildrpmrel}.img initrd-%{ktag}.img
popd > /dev/null

%postun -n %{kname}-%{buildrel}
/sbin/kernel_remove_initrd %{buildrel}
rm -rf /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel} >/dev/null
%endif # build_kernel


### kernel-devel
%if %build_devel
%post -n %{kname}-devel-%{buildrel}
# place /build and /source symlinks in place.
if [ -d /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel} ]; then
    ln -sf /usr/src/linux-%{kversion}-%{ktag}-devel-%{buildrpmrel} \
           /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/build
    ln -sf /usr/src/linux-%{kversion}-%{ktag}-devel-%{buildrpmrel} \
           /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/source
fi

%preun -n %{kname}-devel-%{buildrel}
# we need to delete <modules>/{build,source} at uninstall
if [ -L /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/build ]; then
    rm -f /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/build
fi
if [ -L /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/source ]; then
    rm -f /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/source
fi
exit 0
%endif #build_devel


### kernel-source
%if %build_source
%post -n %{kname}-source-%{buildrel}
if [ -d "/lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}" ]; then
	if [ ! -L $i/build -a ! -L $i/source ]; then
		rm -f $i/{build,source}
	        ln -sf /usr/src/linux-%{kversion}-%{ktag}-%{buildrpmrel} \
	               $i/build
	        ln -sf /usr/src/linux-%{kversion}-%{ktag}-%{buildrpmrel} \
	               $i/source
	fi
fi

%preun -n %{kname}-source-%{buildrel}
for i in /lib/modules/%{kversion}-%{ktag}-%{buildrpmrel}/{build,source}; do
	if [ -L $i ]; then
		if [ "$(readlink $i)" = "/usr/src/linux-%{kversion}-%{ktag}-%{buildrpmrel}" ]; then
			rm -f $i
		fi
	fi
done
exit 0
%endif # build_source


###
### file lists
###

# kernel
%if %build_kernel
%files -n %{kname}-%{buildrel}
%{_bootdir}/config-%{kversion}-%{ktag}-%{buildrpmrel}
%{_bootdir}/vmlinuz-%{kversion}-%{ktag}-%{buildrpmrel}
%{_bootdir}/System.map-%{kversion}-%{ktag}-%{buildrpmrel}
%dir %{_modulesdir}/%{kversion}-%{ktag}-%{buildrpmrel}/
%{_modulesdir}/%{kversion}-%{ktag}-%{buildrpmrel}/kernel
%{_modulesdir}/%{kversion}-%{ktag}-%{buildrpmrel}/modules.*
%doc README.kernel-sources
%endif # build_kernel

# kernel-source
%if %build_source
%files -n %{kname}-source-%{buildrel}
%dir %{_kerneldir}
%dir %{_kerneldir}/arch
%dir %{_kerneldir}/include
%{_kerneldir}/.gitignore
%{_kerneldir}/COPYING
%{_kerneldir}/CREDITS
%{_kerneldir}/Documentation
%{_kerneldir}/Kbuild
%{_kerneldir}/Kconfig
%{_kerneldir}/MAINTAINERS
%{_kerneldir}/Makefile
%{_kerneldir}/README
%{_kerneldir}/REPORTING-BUGS
%{_kerneldir}/arch/Kconfig
%ifarch %{ix86} x86_64
%{_kerneldir}/arch/x86
%endif
%{_kerneldir}/arch/um
%{_kerneldir}/block
%{_kerneldir}/crypto
%{_kerneldir}/drivers
%{_kerneldir}/firmware
%{_kerneldir}/fs
%{_kerneldir}/include/Kbuild
%{_kerneldir}/include/acpi
%{_kerneldir}/include/asm-generic
%{_kerneldir}/include/crypto
%{_kerneldir}/include/clocksource
%{_kerneldir}/include/drm
%{_kerneldir}/include/dt-bindings
%{_kerneldir}/include/linux
%{_kerneldir}/include/math-emu
%{_kerneldir}/include/memory
%{_kerneldir}/include/net
%{_kerneldir}/include/pcmcia
%{_kerneldir}/include/ras
%{_kerneldir}/include/scsi
%{_kerneldir}/include/sound
%{_kerneldir}/include/target
%{_kerneldir}/include/trace
%{_kerneldir}/include/uapi
%{_kerneldir}/include/video
%{_kerneldir}/include/media
%{_kerneldir}/include/misc
%{_kerneldir}/include/rxrpc
%{_kerneldir}/include/keys
%{_kerneldir}/include/rdma
%{_kerneldir}/include/xen
%{_kerneldir}/init
%{_kerneldir}/ipc
%{_kerneldir}/kernel
%{_kerneldir}/lib
%{_kerneldir}/mm
%{_kerneldir}/net
%{_kerneldir}/samples
%{_kerneldir}/scripts
%{_kerneldir}/security
%{_kerneldir}/sound
%{_kerneldir}/tools
%{_kerneldir}/usr
%{_kerneldir}/virt
%doc README.kernel-sources
%endif # build_source

# kernel-devel
%if %build_devel
%files -n %{kname}-devel-%{buildrel}
%dir %{_develdir}
%dir %{_develdir}/arch
%dir %{_develdir}/include
%{_develdir}/.config
%{_develdir}/Documentation
%{_develdir}/Kbuild
%{_develdir}/Kconfig
%{_develdir}/Makefile
%{_develdir}/Module.symvers
%{_develdir}/arch/Kconfig
%ifarch %{ix86} x86_64
%{_develdir}/arch/x86
%endif
%{_develdir}/arch/um
%{_develdir}/block
%{_develdir}/crypto
%{_develdir}/drivers
%{_develdir}/firmware
%{_develdir}/fs
%{_develdir}/include/Kbuild
%{_develdir}/include/acpi
%{_develdir}/include/asm-generic
%{_develdir}/include/config
%{_develdir}/include/crypto
%{_develdir}/include/clocksource
%{_develdir}/include/drm
%{_develdir}/include/dt-bindings
%{_develdir}/include/generated
%{_develdir}/include/keys
%{_develdir}/include/linux
%{_develdir}/include/math-emu
%{_develdir}/include/memory
%{_develdir}/include/misc
%{_develdir}/include/net
%{_develdir}/include/pcmcia
%{_develdir}/include/ras
%{_develdir}/include/rdma
%{_develdir}/include/scsi
%{_develdir}/include/sound
%{_develdir}/include/target
%{_develdir}/include/trace
%{_develdir}/include/uapi
%{_develdir}/include/video
%{_develdir}/include/media
%{_develdir}/include/rxrpc
%{_develdir}/include/xen
%{_develdir}/init
%{_develdir}/ipc
%{_develdir}/kernel
%{_develdir}/lib
%{_develdir}/mm
%{_develdir}/net
%{_develdir}/samples
%{_develdir}/scripts
%{_develdir}/security
%{_develdir}/sound
%{_develdir}/tools
%{_develdir}/usr
%{_develdir}/virt
%doc README.kernel-sources
%endif # build_devel


%if %build_doc
%files -n %{kname}-doc
%doc linux-%{tar_ver}/Documentation/*
%endif # build_doc

%if %build_kernel
%files -n %{kname}-latest
%endif # build_kernel

%if %build_source
%files -n %{kname}-source-latest
%endif # build_source

%if %build_devel
%files -n %{kname}-devel-latest
%endif # build_devel


%changelog
* Sat Feb 01 2014 tmb <tmb> 1:3.10.28-0.vs2.3.6.8.1.mga4
+ Revision: 568579
- update to 3.10.28
- x86, x32: Correct invalid use of user timespec in the kernel (CVE-2014-0038)

* Sun Jan 19 2014 tmb <tmb> 1:3.10.27-0.vs2.3.6.8.1.mga4
+ Revision: 566999
- update to 3.10.27

* Fri Dec 20 2013 tmb <tmb> 1:3.10.25-0.vs2.3.6.8.1.mga4
+ Revision: 559234
- show vserver patch version in release
- drop merged xfs CVE fix
- update to 3.10.25
- drop merged kvm CVE fixes

* Sun Dec 15 2013 tmb <tmb> 1:3.10.24-2.mga4
+ Revision: 556867
- kvm: rtc_status.dest_map out-of-bounds access (CVE-2013-4587)
- kvm: division by zero in apic_get_tmcct() (CVE-2013-6367)
- kvm: cross page vapic_addr access (CVE-2013-6368)
- kvm: BUG_ON() in apic_cluster_id() (CVE-2013-6376)
- xfs: underflow bug in xfs_attrlist_by_handle() (CVE-2013-6382)
- xfs: add capability check to free eofblocks ioctl (CVE pending)
- update to 3.10.24

* Thu Dec 05 2013 tmb <tmb> 1:3.10.22-1.mga4
+ Revision: 555436
- update to 3.10.22

* Sat Nov 16 2013 tmb <tmb> 1:3.10.19-1.mga4
+ Revision: 551553
- ALSA: hda - hdmi: Fix reported channel map on common default layouts
- batman-adv: set up network coding packet handlers during module init
- cxgb3: Fix length calculation in write_ofld_wr() on 32-bit architectures
- hyperv-fb: add pci stub
- ip_gre: Fix WCCPv2 header parsing.
- ipv6: ip6_dst_check needs to check for expired dst_entries
- ipv6: reset dst.expires value when clearing expire flag
- net: flow_dissector: fail on evil iph->ihl
- net/mlx4_core: Fix call to __mlx4_unregister_mac
- USB: add new zte 3g-dongle's pid to option.c
- virtio-net: correctly handle cpu hotplug notifier during resuming
- xen-netback: Handle backend state transitions in a more robust way
- xen-netback: transition to CLOSED when removing a VIF
- xen-netback: use jiffies_64 value to calculate credit timeout
- update to -vs2.3.6.8
- update to 3.10.19

* Mon Nov 04 2013 tmb <tmb> 1:3.10.18-1.mga4
+ Revision: 549476
- cpufreq: ondemand: Change the calculation of target frequency
- update to 3.10.18

* Mon Oct 14 2013 tmb <tmb> 1:3.10.16-1.mga4
+ Revision: 497381
- update to 3.10.16

* Fri Oct 04 2013 tmb <tmb> 1:3.10.15-1.mga4
+ Revision: 491137
- BR: bc
- update filelists
- sync requires and conflicts with core kernel-3.10
- update defconfigs
- update to vs2.3.6.6
- update to 3.10.15

* Sun Apr 07 2013 tmb <tmb> 1:3.7.10-1.mga3
+ Revision: 409021
- fix filelists
- update defconfigs
- update to -vs2.3.5.6
- downgrade to 3.7.10 (latest upstream supported)
- add alx buildfixes (P252, P253)
- update to 3.8.6

* Thu Nov 01 2012 tmb <tmb> 1:3.6.5-1.mga3
+ Revision: 312317
- use correct defconfigs
- update defconfigs
- ext4: fix unjournaled inode bitmap modification
  (P200, fixes possible data corruption bug)
- md/raid1: Fix assembling of arrays containing Replacements
  (P201, fixes reduced data safety)
- net: add new QCA alx ethernet driver (P250, P251)
- update filelists
- update vserver patch to vs2.3.4.3-noxfs-nocow
- update to 3.6.5
- move -doc to Documentation group

* Sun Sep 16 2012 tmb <tmb> 1:3.5.4-1.mga3
+ Revision: 294544
- update vs patch to 2.3.4.2
- update to 3.5.4

* Mon Aug 27 2012 tmb <tmb> 1:3.5.3-1.mga3
+ Revision: 284470
- update to 3.5.3

* Thu Aug 16 2012 tmb <tmb> 1:3.5.2-1.mga3
+ Revision: 281694
- update to 3.5.2
- update vserver patch to 2.3.4.1
- update defconfigs
- update filelists
- enable EFI_STUB support (#6598)

* Wed Jun 27 2012 tmb <tmb> 1:3.3.6-2.mga3
+ Revision: 264360
- rebuild with gcc-4.7.1
- BR kmod instead of module-init-tools
- require kmod instead of module-init-tools

* Sat May 12 2012 tmb <tmb> 1:3.3.6-1.mga2
+ Revision: 235495
- update to 3.3.6
- rediff vs patch
- update defconfigs

* Thu May 10 2012 tmb <tmb> 1:3.3.5-1.mga2
+ Revision: 235079
- update to 3.3.5

* Sat Apr 28 2012 tmb <tmb> 1:3.3.4-1.mga2
+ Revision: 234019
- update to 3.3.4
- require dracut >= 017-9

  + bertl <bertl>
    - updated mga release
    - updated to upstream vs2.3.3.3

* Mon Apr 23 2012 tmb <tmb> 1:3.3.3-1.mga2
+ Revision: 232862
- update to 3.3.3

* Tue Apr 03 2012 bertl <bertl> 1:3.3.1-1.mga2
+ Revision: 228271
- added syscall scripts and c6x arch removal
- updated to latest upstream kernel

  + tmb <tmb>
    - require dracut >= 017-1
    - compress modules with xz

* Thu Mar 01 2012 bertl <bertl> 1:3.2.9-1.mga2
+ Revision: 216607
- updated to upstream 3.2.9

* Tue Feb 21 2012 bertl <bertl> 1:3.2.7-1.mga2
+ Revision: 211481
- updated to upstream 3.2.7

* Fri Feb 17 2012 bertl <bertl> 1:3.2.6-1.mga2
+ Revision: 210023
- updated to 3.2.6-vs2.3.2.7

* Thu Jan 26 2012 tmb <tmb> 1:3.2.2-1.mga2
+ Revision: 201941
- fix SUBLEVEL check
- update filelists
- update defconfigs
- update vserver patch to vs2.3.2.6
- update to 3.2.2
- drop defattr and buildroot
- require module-init-tools >= 3.16-14 for xz support
- require dracut instead of mkinitrd

* Tue Oct 25 2011 tmb <tmb> 1:3.0.8-1.mga2
+ Revision: 158009
- update to 3.0.8
- rediff -cs patch
- drop P200 (merged)

* Wed Oct 19 2011 tmb <tmb> 1:3.0.7-1.mga2
+ Revision: 156608
- update to 3.0.7
- update vs patch to 2.3.1 final
- re-enable usblp as it is needed by both usb-pp adapters and some printers (mga #2240, #2264)
  (cups is patched to work with both usblp and libusb)

* Wed Oct 05 2011 tmb <tmb> 1:3.0.6-1.mga2
+ Revision: 151951
- fix boot hang on uvc webcam init (mga #2425)
- update to 3.0.6
- update vserver patch to vs2.3.1-pre10.1

* Tue Aug 30 2011 tmb <tmb> 1:3.0.4-1.mga2
+ Revision: 136497
- update to 3.0.4
- rediff vserver patch
- update to 3.0.3
- update vserver patch to vs2.3.1-pre9
- adapt spec for 3.0 series kernels
- update defconfigs
- convert -source rpm to noarch
- disable SOUND_OSS_CORE_PRECLAIM to allow osspd to work (request by Coling Guthrie)

* Fri Jul 15 2011 bertl <bertl> 1:2.6.38.8-1.mga2
+ Revision: 124413
- updated to mainline 2.6.38.8 and vs2.3.0.37-rc17

* Sun May 22 2011 tmb <tmb> 1:2.6.38.7-1.mga1
+ Revision: 100107
- rediff vserver patch to apply cleanly
- update to 2.6.38.7 (CVE-2011-1770, CVE-2011-1776, CVE-2011-1927)

* Tue May 10 2011 tmb <tmb> 1:2.6.38.6-1.mga1
+ Revision: 96898
- update to 2.6.38.6
- clean /lib/modules tree on uninstall
- add epoch to support upgrade from mdv

* Wed May 04 2011 tmb <tmb> 2.6.38.5-1.mga1
+ Revision: 94543
- update to 2.6.38.5

* Sat Apr 30 2011 tmb <tmb> 2.6.38.4-1.mga1
+ Revision: 93998
- fix kvserver_notice define
- fix summarys and descriptions
- fix versioning used in builds/paths so it can coexist with kernel-linus
- imported package kernel-vserver

